import QUB
from Buttons import *
from Dialogs import *
from qubtree import Node, NullNode
from qubtree import QTR_TYPE_FLOAT
from math import *
from itertools import *
import Reports
import WordReport
import os


def children(tree, name=None):
    if name == None:
        child = tree.child
    else:
        child = tree.find(name)
    while not child.isNull:
        yield child
        if name == None:
            child = child.sibling
        else:
            child = child.nextSameName()


def Cleave(props):
    try:
        data = QUB.DataView.currData
        lists = data.lists
        list = lists.currList
    except:
        QUB.Report('No selection list available to Cleave.')
        return
    ltree = list.asTree
    ltree.remove(ltree['Sels'])
    lname = str(QUB.getChosenData()['DataDescription'].data)[1:-1]

    odds = Node('List')
    odds.data = "%s cell" % lname
    evens = Node('List')
    evens.data = "%s glass" % lname
    subs = [odds, evens]
    sels = [NullNode(), NullNode()]
    target = 0
    i = 0

    for sel in children(ltree):
        if sel.data.count == 2:
            sels[target] = subs[target].insert(sel.name, sels[target])
            sels[target].data = sel.data
            target = (target + 1) % 2
            i += 1

    for sub in subs:
        data.setList(str(sub.data), [])
        data.chooseList(str(sub.data))
        lists.currList.setFromTree(sub)

AddTaskButton( 'Cleave', # button text
               Cleave,   # function
               'Clv',       # short text for small buttons
               0,            # show settings dialog on left-click?
               0,            # show settings dialog on right-click?
               "Split the selection list into odd and even",   # tool-tip
               "Split the selection list into odd and even",   # description in "Edit Buttons"
               "Cleave settings:", # caption for the settings dialog
               [],
               'Preprocessing' )

def Spew(props):
    try:
        dataFile = QUB.DataView.currData
        data = QUB.getChosenData()
        listname = str(data['DataDescription'].data)[1:-1]
        dataset = data.find('DataSet')
        sampling = dataset['sampling'].data[0] * 1e-3
        headers = ['Time'] + [str(chan['Name'].data) for chan in children(dataset['Channels'], 'Channel')]
        header = '\t'.join(headers) + '\n'
    except:
        QUB.Report('No data available to Spew.')
        return

    path = props["path"].strip() or os.path.join(os.path.split(str(dataFile.fileName))[0],
                                               listname)
    try:
        if not os.path.exists(path):
            os.makedirs(path)
    except:
        QUB.Report("Can't create output folder %s" % path)
        return

    wiz = QUB.CreateWizard()
    wiz.caption = 'Spewing...'
    wiz.show()

    segnum = 0
    segname = lambda: os.path.join(path, "%s%.5d.txt" % (listname, segnum))
    for seg in children(dataset, "Segment"):
        segnum += 1
        start = seg["start"].data[0] * 1e-3
        nr = seg.data[1] - seg.data[0] + 1
        nc = len(headers) - 1 # data.cols
        # workaround for bug in QUB.fillDataCh:
        series = []
        for c in xrange(nc):
            dataset['ActiveChannel'].data = c
            QUB.fillData(seg)
            chan = seg['Channel']
            series.append(chan.data[:])
            seg.remove(chan)
        try:
            wiz.message = 'Writing %s...' % segname()
            out = open(segname(), "w")
            out.write(header)
            for r,vals in enumerate(izip(*series)):
                out.write("%f\t%s\n" % (start+r*sampling, '\t'.join(map(str,vals))))
            out.close()
        except Exception, e:
            QUB.Report('Error saving %s: %s' % (segname(), e))

    wiz.hide()    
    QUB.Report('Spew Done.')
    
    
AddTaskButton( 'Spew', # button text
               Spew,   # function
               'Spew',       # short text for small buttons
               1,            # show settings dialog on left-click?
               1,            # show settings dialog on right-click?
               "Export to a folder of text files",   # tool-tip
               "Export to a folder of text files",   # description in "Edit Buttons"
               "Spew settings:", # caption for the settings dialog
               [DialogText("path", "Output folder (blank for default):", "C:\\spew", acceptString)],
               'Preprocessing' )
